--=====================================================================================
-- FOG GONE SCRIPT
--
-- @Author:		akuenzi
-- @Date:		2019-02-14
-- @Purpose:	To allow player to turn fog on/off in-game.
--=====================================================================================


FogGone = {}

function FogGone:loadMap(name)
	self.firstRun = true;
	self.updateFog = FogGone.updateFog;
	self.adaptTime = FogGone.adaptTime;
	
	self.toggle = true;									-- true means fog is off / false means fog is on.
	
end;

function FogGone:deleteMap()
end;

function FogGone:mouseEvent(posX, posY, modifier, isDown)
end;

function FogGone:keyEvent(unicode, sym, modifier, isDown)
end;

function FogGone:update(dt)

	-- Grab map's environment data on first run.
	if self.firstRun then
		
		--[[Detect whether or not Seasons mod is present.  Seasons adjusts the fog curves every single day.
			If seasons mod is present, then adjust one of its functions to deal with fog stuff for this script.]]
		if g_seasons ~= nil then
			g_seasons.daylight.adaptTime = Utils.appendedFunction(g_seasons.daylight.adaptTime, FogGone.adaptTime);
		end;
		
		-- Store initial fog density values with 'O' prefix.
		local fogCurve = g_currentMission.environment.distanceFogCurve.keyframes;
		for i = 1, table.getn(fogCurve) do
			g_currentMission.environment.distanceFogCurve.keyframes[i].Ow = fogCurve[i].w;
		end;
		
		-- Store initial fog density values for rain types in the '5th' table position.  Default fog densities are specified in the 4th position.
		local rainFog = g_currentMission.environment.rainFogColor;
		for rainType, _ in pairs(rainFog) do
			g_currentMission.environment.rainFogColor[rainType][5] = rainFog[rainType][4];
		end;
		
		-- Turn off the fog to start with.
		self:updateFog();
		
		-- First run at startup is complete.
		self.firstRun = false;
	end;

	-- What to do when input bindings change.
	if InputBinding.hasEvent(InputBinding.Toggle) then
		self.toggle = not self.toggle;
		self:updateFog();
	end;

end;

function FogGone:draw()
end;

function FogGone:adaptTime()

	--[[Seasons mod changes the distanceFogCurve every day.  Since we need to know what the original
		fog curve values are, the code below is appended into a Seasons mod function and should run
		each time the fog curve is updated, so we can get the new 'backup' amounts.]]
	local fogCurve = g_currentMission.environment.distanceFogCurve.keyframes;
	for i = 1, table.getn(fogCurve) do
		fogCurve[i].Ow = fogCurve[i].w;
	end;

end;

function FogGone:updateFog()

	local distanceFogCurve = g_currentMission.environment.distanceFogCurve.keyframes;
	local rainFog = g_currentMission.environment.rainFogColor;
	if self.toggle then
		-- The toggle is 'true.'  Remove fog densities.
		for i = 1, table.getn(distanceFogCurve) do
			g_currentMission.environment.distanceFogCurve.keyframes[i].w = 0;
		end;
		for rainType, _ in pairs(rainFog) do
			g_currentMission.environment.rainFogColor[rainType][4] = 0;
		end;
	else
		-- The toggle is 'false.'  Restore fog densities.
		for i = 1, table.getn(distanceFogCurve) do
			g_currentMission.environment.distanceFogCurve.keyframes[i].w = distanceFogCurve[i].Ow;
		end;
		for rainType, _ in pairs(rainFog) do
			g_currentMission.environment.rainFogColor[rainType][4] = rainFog[rainType][5];
		end;
	end;

end;

addModEventListener(FogGone);